@@ -1,6 +1,5 @@
 {-# LANGUAGE CPP #-}
 {-# LANGUAGE RecordWildCards #-}
-{-# LANGUAGE DeriveDataTypeable #-}
 {-# LANGUAGE FlexibleInstances #-}
 {-# LANGUAGE TypeFamilies #-}
 {-# LANGUAGE TypeSynonymInstances #-}
@@ -23,6 +22,7 @@ module Test.Hspec.Core.Example (
 , safeEvaluateResultStatus
 , exceptionToResultStatus
 , toLocation
+, hunitFailureToResult
 ) where
 
 import           Prelude ()
@@ -33,15 +33,11 @@ import qualified Test.HUnit.Lang as HUnit
 import           Data.CallStack (SrcLoc(..))
 
 import           Control.DeepSeq
-import           Data.Typeable (Typeable)
 import qualified Test.QuickCheck as QC
 import           Test.Hspec.Expectations (Expectation)
 
-import qualified Test.QuickCheck.State as QC (numSuccessTests, maxSuccessTests)
-import qualified Test.QuickCheck.Property as QCP
-
-import           Test.Hspec.Core.QuickCheckUtil
 import           Test.Hspec.Core.Util
+import           Test.Hspec.Core.QuickCheck.Util (liftHook)
 import           Test.Hspec.Core.Example.Location
 
 -- | A type class for examples
@@ -71,25 +67,27 @@ type ActionWith a = a -> IO ()
 data Result = Result {
   resultInfo :: String
 , resultStatus :: ResultStatus
-} deriving (Show, Typeable)
+} deriving Show
 
 data ResultStatus =
     Success
   | Pending (Maybe Location) (Maybe String)
   | Failure (Maybe Location) FailureReason
-  deriving (Show, Typeable)
+  deriving Show
 
 data FailureReason =
     NoReason
   | Reason String
+  | ColorizedReason String
   | ExpectedButGot (Maybe String) String String
   | Error (Maybe String) SomeException
-  deriving (Show, Typeable)
+  deriving Show
 
 instance NFData FailureReason where
   rnf reason = case reason of
     NoReason -> ()
     Reason r -> r `deepseq` ()
+    ColorizedReason r -> r `deepseq` ()
     ExpectedButGot p e a  -> p `deepseq` e `deepseq` a `deepseq` ()
     Error m e -> m `deepseq` show e `deepseq` ()
 
@@ -105,7 +103,7 @@ forceResultStatus r = case r of
   Failure _ m -> m `deepseq` r
 
 safeEvaluateExample :: Example e => e -> Params -> (ActionWith (Arg e) -> IO ()) -> ProgressCallback -> IO Result
-safeEvaluateExample example params around progress = safeEvaluate $ evaluateExample example params around progress
+safeEvaluateExample example params around = safeEvaluate . evaluateExample example params around
 
 safeEvaluate :: IO Result -> IO Result
 safeEvaluate action = do
@@ -182,51 +180,4 @@ toLocation loc = Location (srcLocFile loc) (srcLocStartLine loc) (srcLocStartCol
 
 instance Example (a -> Expectation) where
   type Arg (a -> Expectation) = a
-  evaluateExample e _ hook _ = hook e >> return (Result "" Success)
-
-instance Example QC.Property where
-  type Arg QC.Property = ()
-  evaluateExample e = evaluateExample (\() -> e)
-
-instance Example (a -> QC.Property) where
-  type Arg (a -> QC.Property) = a
-  evaluateExample p c hook progressCallback = do
-    r <- QC.quickCheckWithResult (paramsQuickCheckArgs c) {QC.chatty = False} (QCP.callback qcProgressCallback $ aroundProperty hook p)
-    return $ fromQuickCheckResult r
-    where
-      qcProgressCallback = QCP.PostTest QCP.NotCounterexample $
-        \st _ -> progressCallback (QC.numSuccessTests st, QC.maxSuccessTests st)
-
-fromQuickCheckResult :: QC.Result -> Result
-fromQuickCheckResult r = case parseQuickCheckResult r of
-  QuickCheckResult _ info (QuickCheckOtherFailure err) -> Result info $ Failure Nothing (Reason err)
-  QuickCheckResult _ info QuickCheckSuccess -> Result info Success
-  QuickCheckResult n info (QuickCheckFailure QCFailure{..}) -> case quickCheckFailureException of
-    Just e | Just result <- fromException e -> Result info result
-    Just e | Just hunit <- fromException e -> Result info $ hunitFailureToResult (Just hunitAssertion) hunit
-    Just e -> failure (uncaughtException e)
-    Nothing -> failure falsifiable
-    where
-      failure = Result info . Failure Nothing . Reason
-
-      numbers = formatNumbers n quickCheckFailureNumShrinks
-
-      hunitAssertion :: String
-      hunitAssertion = intercalate "\n" [
-          "Falsifiable " ++ numbers ++ ":"
-        , indent (unlines quickCheckFailureCounterexample)
-        ]
-
-      uncaughtException e = intercalate "\n" [
-          "uncaught exception: " ++ formatException e
-        , numbers
-        , indent (unlines quickCheckFailureCounterexample)
-        ]
-
-      falsifiable = intercalate "\n" [
-          quickCheckFailureReason ++ " " ++ numbers ++ ":"
-        , indent (unlines quickCheckFailureCounterexample)
-        ]
-
-indent :: String -> String
-indent = intercalate "\n" . map ("  " ++) . lines
+  evaluateExample e _params hook _ = hook e >> return (Result "" Success)
