@@ -1,10 +1,17 @@
 {-# LANGUAGE RankNTypes #-}
 {-# LANGUAGE ExistentialQuantification #-}
 -- |
--- Stability: experimental
-module Test.Hspec.Core.Format (
+-- Stability: unstable
+--
+-- This is an unstable API.  Use
+-- [Test.Hspec.Api.Format.V2](https://hackage.haskell.org/package/hspec-api/docs/Test-Hspec-Api-Format-V2.html)
+-- instead.
+module Test.Hspec.Core.Format
+-- {-# WARNING "Use [Test.Hspec.Api.Format.V2](https://hackage.haskell.org/package/hspec-api/docs/Test-Hspec-Api-Format-V2.html) instead." #-}
+(
   Format
 , FormatConfig(..)
+, defaultFormatConfig
 , Event(..)
 , Progress
 , Path
@@ -25,7 +32,7 @@ import qualified Control.Concurrent.Async as Async
 import           Control.Monad.IO.Class
 
 import           Test.Hspec.Core.Example (Progress, Location(..), FailureReason(..))
-import           Test.Hspec.Core.Util (Path)
+import           Test.Hspec.Core.Util (Path, formatExceptionWith)
 import           Test.Hspec.Core.Clock (Seconds(..))
 
 type Format = Event -> IO ()
@@ -60,13 +67,37 @@ data FormatConfig = FormatConfig {
 , formatConfigUseDiff :: Bool
 , formatConfigDiffContext :: Maybe Int
 , formatConfigExternalDiff :: Maybe (String -> String -> IO ())
-, formatConfigPrettyPrint :: Bool -- ^ Deprecated: use `formatConfigPrettyPrintFunction` instead
+, formatConfigPrettyPrint :: Bool
 , formatConfigPrettyPrintFunction :: Maybe (String -> String -> (String, String))
+, formatConfigFormatException :: SomeException -> String -- ^ @since 2.11.5
 , formatConfigPrintTimes :: Bool
 , formatConfigHtmlOutput :: Bool
 , formatConfigPrintCpuTime :: Bool
 , formatConfigUsedSeed :: Integer
 , formatConfigExpectedTotalCount :: Int
+, formatConfigExpertMode :: Bool
+}
+
+{-# DEPRECATED formatConfigPrettyPrint "Use `formatConfigPrettyPrintFunction` instead" #-}
+
+-- ^ @since 2.11.5
+defaultFormatConfig :: FormatConfig
+defaultFormatConfig = FormatConfig {
+  formatConfigUseColor = False
+, formatConfigReportProgress = False
+, formatConfigOutputUnicode = False
+, formatConfigUseDiff = False
+, formatConfigDiffContext = Nothing
+, formatConfigExternalDiff = Nothing
+, formatConfigPrettyPrint = False
+, formatConfigPrettyPrintFunction = Nothing
+, formatConfigFormatException = formatExceptionWith show
+, formatConfigPrintTimes = False
+, formatConfigHtmlOutput = False
+, formatConfigPrintCpuTime = False
+, formatConfigUsedSeed = 0
+, formatConfigExpectedTotalCount = 0
+, formatConfigExpertMode = False
 }
 
 data Signal = Ok | NotOk SomeException
