@@ -1,3 +1,4 @@
+{-# OPTIONS_GHC -fno-warn-deprecations #-}
 {-# LANGUAGE CPP #-}
 {-# LANGUAGE BangPatterns #-}
 {-# LANGUAGE ScopedTypeVariables #-}
@@ -5,6 +6,17 @@
 -- |
 -- Stability: provisional
 module Test.Hspec.Core.Runner (
+-- * Simple interface
+  hspec
+, hspecWith
+, hspecResult
+, hspecWithResult
+
+-- ** Summary
+, Summary (..)
+, isSuccess
+, evaluateSummary
+
 -- * Running a spec
 {- |
 To run a spec `hspec` performs a sequence of steps:
@@ -17,7 +29,8 @@ To run a spec `hspec` performs a sequence of steps:
 The four primitives `evalSpec`, `readConfig`, `runSpecForest` and
 `evaluateResult` each perform one of these steps respectively.
 
-`hspec` is defined in terms of these primitives:
+`hspec` is defined in terms of these primitives. Loosely speaking, a definition
+for @hspec@ is:
 
 @
 hspec = `evalSpec` `defaultConfig` >=> \\ (config, spec) ->
@@ -27,14 +40,36 @@ hspec = `evalSpec` `defaultConfig` >=> \\ (config, spec) ->
   >>= `evaluateResult`
 @
 
-If you need more control over how a spec is run use these primitives individually.
+Loosely speaking in the sense that this definition of @hspec@ ignores
+@--rerun-all-on-success@.
+
+Using these primitives individually gives you more control over how a spec is
+run.  However, if you need support for @--rerun-all-on-success@ then you should
+try hard to solve your use case with one of `hspec`, `hspecWith`, `hspecResult`
+or `hspecWithResult`.
 
 -}
-  hspec
 , evalSpec
 , runSpecForest
 , evaluateResult
 
+-- ** Result
+
+-- *** Spec Result
+, Test.Hspec.Core.Runner.Result.SpecResult
+, Test.Hspec.Core.Runner.Result.specResultItems
+, Test.Hspec.Core.Runner.Result.specResultSuccess
+, toSummary
+
+-- *** Result Item
+, Test.Hspec.Core.Runner.Result.ResultItem
+, Test.Hspec.Core.Runner.Result.resultItemPath
+, Test.Hspec.Core.Runner.Result.resultItemStatus
+, Test.Hspec.Core.Runner.Result.resultItemIsFailure
+
+-- *** Result Item Status
+, Test.Hspec.Core.Runner.Result.ResultItemStatus(..)
+
 -- * Config
 , Config (..)
 , ColorMode (..)
@@ -46,35 +81,9 @@ If you need more control over how a spec is run use these primitives individuall
 , configAddFilter
 , readConfig
 
--- * Result
-
--- ** Spec Result
-, Test.Hspec.Core.Runner.Result.SpecResult
-, Test.Hspec.Core.Runner.Result.specResultItems
-, Test.Hspec.Core.Runner.Result.specResultSuccess
-
--- ** Result Item
-, Test.Hspec.Core.Runner.Result.ResultItem
-, Test.Hspec.Core.Runner.Result.resultItemPath
-, Test.Hspec.Core.Runner.Result.resultItemStatus
-, Test.Hspec.Core.Runner.Result.resultItemIsFailure
-
--- ** Result Item Status
-, Test.Hspec.Core.Runner.Result.ResultItemStatus(..)
-
 -- * Legacy
--- | The following primitives are deprecated.  Use `runSpecForest` instead.
-, hspecWith
-, hspecResult
-, hspecWithResult
 , runSpec
 
--- ** Summary
-, Summary (..)
-, toSummary
-, isSuccess
-, evaluateSummary
-
 -- * Re-exports
 , Spec
 , SpecWith
@@ -106,16 +115,17 @@ import qualified Test.QuickCheck as QC
 import           Test.Hspec.Core.Util (Path)
 import           Test.Hspec.Core.Clock
 import           Test.Hspec.Core.Spec hiding (pruneTree, pruneForest)
+import           Test.Hspec.Core.Tree (formatDefaultDescription)
 import           Test.Hspec.Core.Config
 import           Test.Hspec.Core.Format (Format, FormatConfig(..))
 import qualified Test.Hspec.Core.Formatters.V1 as V1
 import qualified Test.Hspec.Core.Formatters.V2 as V2
 import           Test.Hspec.Core.FailureReport
-import           Test.Hspec.Core.QuickCheckUtil
+import           Test.Hspec.Core.QuickCheck.Util
 import           Test.Hspec.Core.Shuffle
 
 import           Test.Hspec.Core.Runner.PrintSlowSpecItems
-import           Test.Hspec.Core.Runner.Eval hiding (Tree(..))
+import           Test.Hspec.Core.Runner.Eval hiding (ColorMode(..), Tree(..))
 import qualified Test.Hspec.Core.Runner.Eval as Eval
 import           Test.Hspec.Core.Runner.Result
 
@@ -124,6 +134,7 @@ import           Test.Hspec.Core.Runner.Result
 --
 -- @since 2.10.5
 registerFormatter :: (String, FormatConfig -> IO Format) -> Config -> Config
+{-# DEPRECATED registerFormatter "Use [@registerFormatter@](https://hackage.haskell.org/package/hspec-api/docs/Test-Hspec-Api-Format-V2.html#v:registerFormatter) instead." #-}
 registerFormatter formatter config = config { configAvailableFormatters = formatter : configAvailableFormatters config }
 
 -- |
@@ -131,6 +142,7 @@ registerFormatter formatter config = config { configAvailableFormatters = format
 --
 -- @since 2.10.5
 registerDefaultFormatter :: (String, FormatConfig -> IO Format) -> Config -> Config
+{-# DEPRECATED registerDefaultFormatter "Use [@useFormatter@](https://hackage.haskell.org/package/hspec-api/docs/Test-Hspec-Api-Format-V2.html#v:useFormatter) instead." #-}
 registerDefaultFormatter formatter@(_, format) config = (registerFormatter formatter config) { configFormat = Just format }
 
 applyFilterPredicates :: Config -> [Tree c EvalItem] -> [Tree c EvalItem]
@@ -183,12 +195,12 @@ evalSpec config spec = do
 
 -- Add a seed to given config if there is none.  That way the same seed is used
 -- for all properties.  This helps with --seed and --rerun.
-ensureSeed :: Config -> IO Config
-ensureSeed c = case configQuickCheckSeed c of
-  Nothing -> do
-    seed <- newSeed
-    return c {configQuickCheckSeed = Just (fromIntegral seed)}
-  _       -> return c
+ensureSeed :: Config -> IO (Config, Integer)
+ensureSeed config = do
+  seed <- case configSeed config <|> configQuickCheckSeed config of
+    Nothing -> toInteger <$> newSeed
+    Just seed -> return seed
+  return (config { configSeed = Just seed }, seed)
 
 -- | Run given spec with custom options.
 -- This is similar to `hspec`, but more flexible.
@@ -277,26 +289,48 @@ runSpecForest spec config = do
 mapItem :: (Item a -> Item b) -> [SpecTree a] -> [SpecTree b]
 mapItem f = map (fmap f)
 
+mapItemIf :: (Item a -> Bool) -> (Item a -> Item a) -> [SpecTree a] -> [SpecTree a]
+mapItemIf p f = mapItem $ \ item -> if p item then f item else item
+
+addDefaultDescriptions :: [SpecTree a] -> [SpecTree a]
+addDefaultDescriptions = mapItem addDefaultDescription
+  where
+    addDefaultDescription :: Item a -> Item a
+    addDefaultDescription item
+      | null (itemRequirement item) = item { itemRequirement = defaultRequirement }
+      | otherwise = item
+      where
+        defaultRequirement = maybe "(unspecified behavior)" formatDefaultDescription (itemLocation item)
+
+failItemsWithEmptyDescription :: Config -> [SpecTree a] -> [SpecTree a]
+failItemsWithEmptyDescription config
+  | configFailOnEmptyDescription config = mapItemIf condition (failWith failure)
+  | otherwise = id
+  where
+    condition = null . itemRequirement
+    failure = "item has no description; failing due to --fail-on=empty-description"
+
 failFocusedItems :: Config -> [SpecTree a] -> [SpecTree a]
 failFocusedItems config
-  | configFailOnFocused config = mapItem failFocused
+  | configFailOnFocused config = mapItemIf condition (failWith failure)
   | otherwise = id
+  where
+    condition = itemIsFocused
+    failure = "item is focused; failing due to --fail-on=focused"
 
-failFocused :: forall a. Item a -> Item a
-failFocused item = item {itemExample = example}
+failWith :: forall a. String -> Item a -> Item a
+failWith reason item = item {itemExample = example}
   where
     failure :: ResultStatus
-    failure = Failure Nothing (Reason "item is focused; failing due to --fail-on=focused")
+    failure = Failure Nothing (Reason reason)
 
     example :: Params -> (ActionWith a -> IO ()) -> ProgressCallback -> IO Result
-    example
-      | itemIsFocused item = \ params hook p -> do
-          Result info status <- itemExample item params hook p
-          return $ Result info $ case status of
-            Success -> failure
-            Pending _ _ -> failure
-            Failure{} -> status
-      | otherwise = itemExample item
+    example params hook p = do
+      Result info status <- itemExample item params hook p
+      return $ Result info $ case status of
+        Success -> failure
+        Pending _ _ -> failure
+        Failure{} -> status
 
 failPendingItems :: Config -> [SpecTree a] -> [SpecTree a]
 failPendingItems config
@@ -321,14 +355,13 @@ focusSpec config spec
 runSpecForest_ :: Maybe FailureReport -> [SpecTree ()] -> Config -> IO SpecResult
 runSpecForest_ oldFailureReport spec c_ = do
 
-  config <- ensureSeed (applyFailureReport oldFailureReport c_)
+  (config, seed) <- ensureSeed (applyFailureReport oldFailureReport c_)
 
   colorMode <- colorOutputSupported (configColorMode config) (hSupportsANSI stdout)
   outputUnicode <- unicodeOutputSupported (configUnicodeMode config) stdout
 
   let
-    filteredSpec = specToEvalForest config spec
-    seed = (fromJust . configQuickCheckSeed) config
+    filteredSpec = specToEvalForest seed config spec
     qcArgs = configQuickCheckArgs config
     !numberOfItems = countEvalItems filteredSpec
 
@@ -349,11 +382,13 @@ runSpecForest_ oldFailureReport spec c_ = do
       , formatConfigExternalDiff = if configDiff config then ($ configDiffContext config) <$> configExternalDiff config else Nothing
       , formatConfigPrettyPrint = configPrettyPrint config
       , formatConfigPrettyPrintFunction = if configPrettyPrint config then Just (configPrettyPrintFunction config outputUnicode) else Nothing
+      , formatConfigFormatException = configFormatException config
       , formatConfigPrintTimes = configTimes config
       , formatConfigHtmlOutput = configHtmlOutput config
       , formatConfigPrintCpuTime = configPrintCpuTime config
       , formatConfigUsedSeed = seed
       , formatConfigExpectedTotalCount = numberOfItems
+      , formatConfigExpertMode = configExpertMode config
       }
 
       formatter = fromMaybe (V2.formatterToFormat V2.checks) (configFormat config <|> V1.formatterToFormat <$> configFormatter config)
@@ -365,6 +400,7 @@ runSpecForest_ oldFailureReport spec c_ = do
         evalConfigFormat = format
       , evalConfigConcurrentJobs = concurrentJobs
       , evalConfigFailFast = configFailFast config
+      , evalConfigColorMode = bool Eval.ColorDisabled Eval.ColorEnabled (shouldUseColor colorMode)
       }
     runFormatter evalConfig filteredSpec
 
@@ -376,9 +412,11 @@ runSpecForest_ oldFailureReport spec c_ = do
 
   return results
 
-specToEvalForest :: Config -> [SpecTree ()] -> [EvalTree]
-specToEvalForest config =
-      failFocusedItems config
+specToEvalForest :: Integer -> Config -> [SpecTree ()] -> [EvalTree]
+specToEvalForest seed config =
+      failItemsWithEmptyDescription config
+  >>> addDefaultDescriptions
+  >>> failFocusedItems config
   >>> failPendingItems config
   >>> focusSpec config
   >>> toEvalItemForest params
@@ -387,9 +425,6 @@ specToEvalForest config =
   >>> randomize
   >>> pruneForest
   where
-    seed :: Integer
-    seed = (fromJust . configQuickCheckSeed) config
-
     params :: Params
     params = Params (configQuickCheckArgs config) (configSmallCheckDepth config)
 
