@@ -1,9 +1,11 @@
+{-# LANGUAGE  ViewPatterns #-}
 -- | Stability: unstable
 module Test.Hspec.Core.Util (
 -- * String functions
   pluralize
 , strip
 , lineBreaksAt
+, stripAnsi
 
 -- * Working with paths
 , Path
@@ -14,6 +16,7 @@ module Test.Hspec.Core.Util (
 -- * Working with exceptions
 , safeTry
 , formatException
+, formatExceptionWith
 ) where
 
 import           Prelude ()
@@ -69,6 +72,18 @@ lineBreaksAt n = concatMap f . lines
           then go (r, ys)
           else s : go (y, ys)
 
+-- |
+-- Remove ANSI color escape sequences.
+--
+-- @since 2.11.0
+stripAnsi :: String -> String
+stripAnsi = go
+  where
+    go input = case input of
+      '\ESC' : '[' : (dropWhile (`elem` "0123456789;") -> 'm' : xs) -> go xs
+      x : xs -> x : go xs
+      [] -> []
+
 -- |
 -- A `Path` describes the location of a spec item within a spec tree.
 --
@@ -123,9 +138,13 @@ filterPredicate pattern path =
 --
 -- @since 2.0.0
 formatException :: SomeException -> String
-formatException err@(SomeException e) = case fromException err of
-  Just ioe -> showType ioe ++ " of type " ++ showIOErrorType ioe ++ "\n" ++ show ioe
-  Nothing  -> showType e ++ "\n" ++ show e
+formatException = formatExceptionWith show
+
+-- | @since 2.11.5
+formatExceptionWith :: (SomeException -> String) -> SomeException -> String
+formatExceptionWith showException err@(SomeException e) = case fromException err of
+  Just ioe -> showType ioe ++ " of type " ++ showIOErrorType ioe ++ "\n" ++ showException (toException ioe)
+  Nothing  -> showType e ++ "\n" ++ showException (SomeException e)
   where
     showIOErrorType :: IOException -> String
     showIOErrorType ioe = case ioe_type ioe of
