@@ -1,16 +1,17 @@
 {-# LANGUAGE CPP #-}
 {-# LANGUAGE OverloadedStrings #-}
+{-# OPTIONS_GHC -fno-warn-deprecations #-}
 module Test.Hspec.Core.Formatters.V1Spec (spec) where
 
 import           Prelude ()
-import           Helper
+import           Helper hiding (colorize)
 import           Data.String
 import           Control.Monad.IO.Class
 import           Control.Monad.Trans.Writer hiding (pass)
 
 import qualified Test.Hspec.Core.Spec as H
 import qualified Test.Hspec.Core.Runner as H
-import qualified Test.Hspec.Core.Formatters.V1 as H
+import qualified Test.Hspec.Core.Formatters.V1 as H hiding (FailureReason(..))
 import qualified Test.Hspec.Core.Formatters.V1.Monad as H (interpretWith)
 import           Test.Hspec.Core.Formatters.V1.Monad (FormatM, Environment(..), FailureRecord(..), FailureReason(..))
 
