@@ -1,3 +1,4 @@
+{-# OPTIONS_GHC -fno-warn-deprecations #-}
 module Test.Hspec.Core.Formatters.V2Spec (spec) where
 
 import           Prelude ()
@@ -19,22 +20,14 @@ testSpec = do
     H.it "exceptions" (undefined :: Spec.Result)
     H.it "fail 3"     (H.Result "" $ Spec.Failure Nothing H.NoReason)
 
-
 formatConfig :: FormatConfig
-formatConfig = FormatConfig {
-  formatConfigUseColor = False
-, formatConfigReportProgress = False
-, formatConfigOutputUnicode = unicode
+formatConfig = defaultFormatConfig {
+  formatConfigOutputUnicode = unicode
 , formatConfigUseDiff = True
 , formatConfigDiffContext = Just 3
 , formatConfigExternalDiff = Nothing
 , formatConfigPrettyPrint = True
 , formatConfigPrettyPrintFunction = Just (H.configPrettyPrintFunction H.defaultConfig unicode)
-, formatConfigPrintTimes = False
-, formatConfigHtmlOutput = False
-, formatConfigPrintCpuTime = False
-, formatConfigUsedSeed = 0
-, formatConfigExpectedTotalCount = 0
 } where
     unicode = True
 
@@ -59,19 +52,46 @@ spec = do
         indentChunks "  " [Modified "foo"] `shouldBe` [ColorChunk "foo"]
 
       it "indents multi-line input" $ do
-        indentChunks "  " [Modified "foo\nbar\nbaz\n"] `shouldBe` [ColorChunk "foo", PlainChunk "\n  ", ColorChunk "bar", PlainChunk "\n  ", ColorChunk "baz", PlainChunk "\n  "]
+        indentChunks "  " [Modified "foo\nbar\nbaz\n"] `shouldBe` [ColorChunk "foo", PlainChunk "\n  ", ColorChunk "bar", PlainChunk "\n  ", ColorChunk "baz", PlainChunk "\n", ColorChunk "  "]
 
       it "colorizes whitespace-only input" $ do
         indentChunks "  " [Modified "  "] `shouldBe` [ColorChunk "  "]
 
       it "colorizes whitespace-only lines" $ do
-        indentChunks "  " [Modified "foo\n  \n"] `shouldBe` [ColorChunk "foo", PlainChunk "\n  ", ColorChunk "  ", PlainChunk "\n  "]
+        indentChunks "  " [Modified "foo\n  \n"] `shouldBe` [ColorChunk "foo", PlainChunk "\n  ", ColorChunk "  ", PlainChunk "\n", ColorChunk "  "]
 
       it "colorizes whitespace at the end of the input" $ do
         indentChunks "  " [Modified "foo\n  "] `shouldBe` [ColorChunk "foo", PlainChunk "\n  ", ColorChunk "  "]
 
       it "splits off whitespace-only segments at the end of a line so that they get colorized" $ do
-        indentChunks "  " [Modified "foo  \n"] `shouldBe` [ColorChunk "foo", ColorChunk "  ", PlainChunk "\n  "]
+        indentChunks "  " [Modified "foo  \n"] `shouldBe` [ColorChunk "foo", ColorChunk "  ", PlainChunk "\n", ColorChunk "  "]
+
+      it "splits off whitespace-only segments at the end of the input so that they get colorized" $ do
+        indentChunks "  " [Modified "23 "] `shouldBe` [ColorChunk "23", ColorChunk " "]
+
+      context "when next chunk starts with a newline" $ do
+        it "splits off whitespace-only segments at the end of a chunk so that they get colorized" $ do
+          indentChunks "  " [Modified "23 ", Original "\nbar"] `shouldBe` [ColorChunk "23", ColorChunk " ", PlainChunk "\n  bar"]
+
+      context "when next chunk starts with spaces followed by a newline" $ do
+        it "splits off whitespace-only segments at the end of a chunk so that they get colorized" $ do
+          indentChunks "  " [Modified "23 ", Original "   \nbar"] `shouldBe` [ColorChunk "23", ColorChunk " ", PlainChunk "   \n  bar"]
+
+      context "when all following chunks only consist of whitespace characters" $ do
+        it "splits off whitespace-only segments at the end of a chunk so that they get colorized" $ do
+          indentChunks "  " [Modified "23 ", Original "  ", Original "  "] `shouldBe` [ColorChunk "23", ColorChunk " ", PlainChunk "  ", PlainChunk "  "]
+
+      context "when all following chunks only consist of whitespace characters until the next newline is encountered" $ do
+        it "splits off whitespace-only segments at the end of a chunk so that they get colorized" $ do
+          indentChunks "  " [Modified "23 ", Original " ", Modified " ", Original "\n"] `shouldBe` [ColorChunk "23", ColorChunk " ", PlainChunk " ", ColorChunk " ", PlainChunk "\n  "]
+
+      context "when next chunk starts with a non-whitespace character" $ do
+        it "does not split off whitespace-only segments at the end of a chunk" $ do
+          indentChunks "  " [Modified "23 ", Original "bar"] `shouldBe` [ColorChunk "23 ", PlainChunk "bar"]
+
+      context "when all following chunks only consist of non-newline characters until the next non-whitespace character is encountered" $ do
+        it "does not split off whitespace-only segments at the end of a chunk" $ do
+          indentChunks "  " [Modified "23 ", Original "  ", Original " bar"] `shouldBe` [ColorChunk "23 ", PlainChunk "  ", PlainChunk " bar"]
 
       context "with empty lines" $ do
         it "colorizes indentation" $ do
@@ -207,9 +227,11 @@ spec = do
           ]
 
     describe "formatterDone" $ do
+      let expectedButGot expected actual = ItemDone ([], "") . Item Nothing 0 "" $ Failure Nothing $ ExpectedButGot Nothing expected actual
+
       it "recovers unicode from ExpectedButGot" $ do
         formatter <- formatterToFormat failed_examples formatConfig { formatConfigOutputUnicode = True }
-        _ <- formatter .  ItemDone ([], "") . Item Nothing 0 "" $ Failure Nothing $ ExpectedButGot Nothing (show "\955") (show "\956")
+        formatter $ expectedButGot (show "\955") (show "\956")
         (fmap normalizeSummary . captureLines) (formatter $ Done []) `shouldReturn` [
             ""
           , "Failures:"
@@ -218,7 +240,7 @@ spec = do
           , "       expected: \"λ\""
           , "        but got: \"μ\""
           , ""
-          , "  To rerun use: --match \"//\""
+          , "  To rerun use: --match \"//\" --seed 0"
           , ""
           , "Randomized with seed 0"
           , ""
@@ -226,10 +248,28 @@ spec = do
           , "1 example, 1 failure"
           ]
 
+      context "on --expert" $ do
+        it "does not print rerun message" $ do
+          formatter <- formatterToFormat failed_examples formatConfig { formatConfigExpertMode = True }
+          formatter $ expectedButGot "foo" "bar"
+          (fmap normalizeSummary . captureLines) (formatter $ Done []) `shouldReturn` [
+              ""
+            , "Failures:"
+            , ""
+            , "  1) "
+            , "       expected: foo"
+            , "        but got: bar"
+            , ""
+            , "Randomized with seed 0"
+            , ""
+            , "Finished in 0.0000 seconds"
+            , "1 example, 1 failure"
+            ]
+
       context "when actual/expected contain newlines" $ do
         it "adds indentation" $ do
           formatter <- formatterToFormat failed_examples formatConfig
-          _ <- formatter .  ItemDone ([], "") . Item Nothing 0 "" $ Failure Nothing $ ExpectedButGot Nothing "first\nsecond\nthird" "first\ntwo\nthird"
+          formatter $ expectedButGot "first\nsecond\nthird" "first\ntwo\nthird"
           (fmap normalizeSummary . captureLines) (formatter $ Done []) `shouldReturn` [
               ""
             , "Failures:"
@@ -242,7 +282,7 @@ spec = do
             , "                 two"
             , "                 third"
             , ""
-            , "  To rerun use: --match \"//\""
+            , "  To rerun use: --match \"//\" --seed 0"
             , ""
             , "Randomized with seed 0"
             , ""
@@ -253,7 +293,7 @@ spec = do
       context "without failures" $ do
         it "shows summary in green if there are no failures" $ do
           formatter <- formatterToFormat failed_examples formatConfig
-          _ <- formatter .  ItemDone ([], "") . Item Nothing 0 "" $ Success
+          formatter . ItemDone ([], "") . Item Nothing 0 "" $ Success
           (fmap normalizeSummary . captureLines) (formatter $ Done []) `shouldReturn` [
               ""
             , "Finished in 0.0000 seconds"
@@ -263,7 +303,7 @@ spec = do
       context "with pending examples" $ do
         it "shows summary in yellow if there are pending examples" $ do
           formatter <- formatterToFormat failed_examples formatConfig
-          _ <- formatter .  ItemDone ([], "") . Item Nothing 0 "" $ Pending Nothing Nothing
+          formatter . ItemDone ([], "") . Item Nothing 0 "" $ Pending Nothing Nothing
           (fmap normalizeSummary . captureLines) (formatter $ Done []) `shouldReturn` [
               ""
             , "Finished in 0.0000 seconds"
