@@ -24,6 +24,7 @@ evalSpec = fmap normalize . (toEvalForest >=> runFormatter config)
       evalConfigFormat = \ _ -> pass
     , evalConfigConcurrentJobs = 1
     , evalConfigFailFast = False
+    , evalConfigColorMode = ColorEnabled
     }
     normalize = map $ \ (path, item) -> (pathToList path, normalizeItem item)
     normalizeItem item = item {
@@ -37,7 +38,7 @@ evalSpec = fmap normalize . (toEvalForest >=> runFormatter config)
     pathToList (xs, x) = xs ++ [x]
 
 toEvalForest :: H.SpecWith () -> IO [EvalTree]
-toEvalForest = fmap (uncurry H.specToEvalForest . first (($ H.defaultConfig) . appEndo)) . H.runSpecM
+toEvalForest = fmap (uncurry (H.specToEvalForest 0) . first (($ H.defaultConfig) . appEndo)) . H.runSpecM
 
 mkAppend :: IO (String -> IO (), IO [String])
 mkAppend = do
@@ -666,17 +667,9 @@ spec = do
     divideByZero = Failure Nothing (Error Nothing $ toException DivideByZero)
 
     divideByZeroIn :: String -> Result
-#if MIN_VERSION_base(4,8,1)
     divideByZeroIn hook = Failure Nothing (Error (Just $ "in " <> hook <> "-hook:") $ toException DivideByZero)
-#else
-    divideByZeroIn _ = Failure Nothing (Error Nothing $ toException DivideByZero)
-#endif
 
     item :: [String] -> Result -> ([String], Item)
     item path result = (path, Item Nothing 0 "" result)
 
-#if MIN_VERSION_base(4,8,1)
     exceptionIn name = Just ("exception in " <> name <> "-hook (see previous failure)")
-#else
-    exceptionIn _ = Just "exception in beforeAll-hook (see previous failure)"
-#endif
