@@ -1,25 +1,19 @@
 {-# LANGUAGE CPP #-}
 {-# LANGUAGE ScopedTypeVariables #-}
 {-# LANGUAGE TypeFamilies #-}
-
-#if MIN_VERSION_base(4,6,0) && !MIN_VERSION_base(4,7,0)
--- Control.Concurrent.QSem is deprecated in base-4.6.0.*
-{-# OPTIONS_GHC -fno-warn-deprecations #-}
-#endif
-
+{-# LANGUAGE FlexibleContexts #-}
+{-# LANGUAGE ConstraintKinds #-}
 module Test.Hspec.Core.RunnerSpec (spec) where
 
 import           Prelude ()
 import           Helper
 
 import           System.IO
-import           System.Environment (withArgs, withProgName, getArgs)
+import           System.Environment (withArgs, withProgName, getArgs, setEnv, unsetEnv)
 import           System.Exit
 import           Control.Concurrent
 import           Control.Concurrent.Async
 import           Mock
-import           System.SetEnv
-import           System.Console.ANSI
 
 import           Test.Hspec.Core.FailureReport (FailureReport(..))
 import qualified Test.Hspec.Expectations as H
@@ -43,6 +37,18 @@ runPropFoo args = unlines . normalizeSummary . lines <$> do
 person :: Int -> Person
 person = Person "Joe"
 
+data MyException = MyException
+  deriving (Eq, Show)
+
+instance Exception MyException where
+  displayException MyException = "my exception"
+
+resultWithColorizedReason :: H.Result
+resultWithColorizedReason = H.Result {
+  H.resultInfo = "info"
+, H.resultStatus = H.Failure Nothing . H.ColorizedReason $ "some " <> green "colorized" <> " error message"
+}
+
 spec :: Spec
 spec = do
   describe "hspec" $ do
@@ -234,7 +240,7 @@ spec = do
           , ""
           , "  1) foo"
           , ""
-          , "  To rerun use: --match \"/foo/\""
+          , "  To rerun use: --match \"/foo/\" --seed 23"
           , ""
           , "Randomized with seed 23"
           , ""
@@ -256,12 +262,12 @@ spec = do
         takeMVar mvar `shouldReturn` UserInterrupt
 
     context "with --dry-run" $ do
-      let withDryRun = captureLines . withArgs ["--dry-run"] . H.hspec
+      let withDryRun = hspecCapture ["--dry-run"]
       it "produces a report" $ do
-        r <- withDryRun $ do
+        withDryRun $ do
           H.it "foo" True
           H.it "bar" True
-        normalizeSummary r `shouldBe` [
+        `shouldReturn` unlines [
             ""
           , "foo [✔]"
           , "bar [✔]"
@@ -285,13 +291,13 @@ spec = do
         readIORef ref `shouldReturn` False
 
     context "with --focused-only" $ do
-      let run = captureLines . withArgs ["--focused-only"] . H.hspec
+      let run = hspecCapture ["--focused-only"]
       context "when there aren't any focused spec items" $ do
         it "does not run anything" $ do
-          r <- run $ do
+          run $ do
             H.it "foo" True
             H.it "bar" True
-          normalizeSummary r `shouldBe` [
+          `shouldReturn` unlines [
               ""
             , ""
             , "Finished in 0.0000 seconds"
@@ -310,12 +316,12 @@ spec = do
         r `shouldBe` Left (ExitFailure 1)
 
     context "with --fail-on=focused" $ do
-      let run = captureLines . ignoreExitCode . withArgs ["--fail-on=focused", "--seed", "23"] . H.hspec . removeLocations
+      let run = hspecCapture ["--fail-on=focused", "--seed", "23"]
       it "fails on focused spec items" $ do
-        r <- run $ do
+        run $ do
           H.it "foo" True
           H.fit "bar" True
-        normalizeSummary r `shouldBe` [
+        `shouldReturn` unlines [
             ""
           , "bar [✘]"
           , ""
@@ -324,7 +330,7 @@ spec = do
           , "  1) bar"
           , "       item is focused; failing due to --fail-on=focused"
           , ""
-          , "  To rerun use: --match \"/bar/\""
+          , "  To rerun use: --match \"/bar/\" --seed 23"
           , ""
           , "Randomized with seed 23"
           , ""
@@ -332,15 +338,38 @@ spec = do
           , "1 example, 1 failure"
           ]
 
+    context "with --fail-on=empty-description" $ do
+      let run = hspecCapture ["--fail-on=empty-description", "--seed", "23"]
+      it "fails on items with empty requirement" $ do
+        run $ do
+          H.it "foo" True
+          H.it "" True
+        `shouldReturn` unlines [
+            ""
+          , "foo [✔]"
+          , "(unspecified behavior) [✘]"
+          , ""
+          , "Failures:"
+          , ""
+          , "  1) (unspecified behavior)"
+          , "       item has no description; failing due to --fail-on=empty-description"
+          , ""
+          , "  To rerun use: --match \"/(unspecified behavior)/\" --seed 23"
+          , ""
+          , "Randomized with seed 23"
+          , ""
+          , "Finished in 0.0000 seconds"
+          , "2 examples, 1 failure"
+          ]
+
     context "with --fail-on=pending" $ do
-      let run = captureLines . ignoreExitCode . withArgs ["--fail-on=pending", "--seed", "23"] . H.hspec . removeLocations
+      let run = hspecCapture ["--fail-on=pending", "--seed", "23"]
       it "fails on pending spec items" $ do
-        r <- run $ do
+        run $ do
           H.it "foo" True
           H.it "bar" $ do
             void $ throwIO (H.Pending Nothing Nothing)
-
-        normalizeSummary r `shouldBe` [
+        `shouldReturn` unlines [
             ""
           , "foo [✔]"
           , "bar [✘]"
@@ -350,7 +379,7 @@ spec = do
           , "  1) bar"
           , "       item is pending; failing due to --fail-on=pending"
           , ""
-          , "  To rerun use: --match \"/bar/\""
+          , "  To rerun use: --match \"/bar/\" --seed 23"
           , ""
           , "Randomized with seed 23"
           , ""
@@ -360,11 +389,11 @@ spec = do
 
     context "with --fail-fast" $ do
       it "stops after first failure" $ do
-        r <- captureLines . ignoreExitCode . withArgs ["--fail-fast", "--seed", "23"] . H.hspec . removeLocations $ do
+        hspecCapture ["--fail-fast", "--seed", "23"] $ do
           H.it "foo" True
           H.it "bar" False
           H.it "baz" False
-        normalizeSummary r `shouldBe` [
+        `shouldReturn` unlines [
             ""
           , "foo [✔]"
           , "bar [✘]"
@@ -373,7 +402,7 @@ spec = do
           , ""
           , "  1) bar"
           , ""
-          , "  To rerun use: --match \"/bar/\""
+          , "  To rerun use: --match \"/bar/\" --seed 23"
           , ""
           , "Randomized with seed 23"
           , ""
@@ -400,11 +429,11 @@ spec = do
         readIORef ref `shouldReturn` ""
 
       it "works for nested specs" $ do
-        r <- captureLines . ignoreExitCode . withArgs ["--fail-fast", "--seed", "23"] . H.hspec . removeLocations $ do
+        hspecCapture ["--fail-fast", "--seed", "23"] $ do
           H.describe "foo" $ do
             H.it "bar" False
             H.it "baz" True
-        normalizeSummary r `shouldBe` [
+        `shouldReturn` unlines [
             ""
           , "foo"
           , "  bar [✘]"
@@ -413,7 +442,7 @@ spec = do
           , ""
           , "  1) foo bar"
           , ""
-          , "  To rerun use: --match \"/foo/bar/\""
+          , "  To rerun use: --match \"/foo/bar/\" --seed 23"
           , ""
           , "Randomized with seed 23"
           , ""
@@ -421,6 +450,26 @@ spec = do
           , "1 example, 1 failure"
           ]
 
+      context "with a cleanup action" $ do
+        it "runs cleanup action" $ do
+          ref <- newIORef 0
+          ignoreExitCode $ hspec ["--fail-fast"] $ do
+            H.afterAll_ (modifyIORef ref succ) $ do
+              H.it "foo" True
+              H.it "bar" False
+              H.it "baz" True
+          readIORef ref `shouldReturn` (1 :: Int)
+
+        context "when last leaf fails" $ do
+          it "runs cleanup action exactly once" $ do
+            ref <- newIORef 0
+            ignoreExitCode $ hspec ["--fail-fast"] $ do
+              H.afterAll_ (modifyIORef ref succ) $ do
+                H.it "foo" True
+                H.it "bar" True
+                H.it "baz" False
+            readIORef ref `shouldReturn` (1 :: Int)
+
     context "with --match" $ do
       it "only runs examples that match a given pattern" $ do
         e1 <- newMock
@@ -485,7 +534,7 @@ spec = do
           , "                   personAge = 23"
           , "                 }"
           , ""
-          , "  To rerun use: --match \"/foo/\""
+          , "  To rerun use: --match \"/foo/\" --seed 0"
           , ""
           , "Randomized with seed 0"
           , ""
@@ -516,14 +565,97 @@ spec = do
           , "        but got: Person {personName = \"Joe\", personAge = 23}"
           ]
 
+    context "when formatting exceptions" $ do
+      let spec_ = H.it "foo" $ void (withFrozenCallStack throwIO MyException)
+      context "with --show-exceptions" $ do
+        it "uses `show`" $ do
+          hspecCapture ["--seed=0", "--format=failed-examples", "--display-exceptions", "--show-exceptions"] spec_
+          `shouldReturn` unlines [
+              ""
+            , "Failures:"
+            , ""
+            , "  1) foo"
+            , "       uncaught exception: MyException"
+            , "       MyException"
+            , ""
+            , "  To rerun use: --match \"/foo/\" --seed 0"
+            , ""
+            , "Randomized with seed 0"
+            , ""
+            , "Finished in 0.0000 seconds"
+            , "1 example, 1 failure"
+            ]
+
+      context "with --display-exceptions" $ do
+        it "uses `displayException`" $ do
+          hspecCapture ["--seed=0", "--format=failed-examples", "--show-exceptions", "--display-exceptions"] spec_
+          `shouldReturn` unlines [
+              ""
+            , "Failures:"
+            , ""
+            , "  1) foo"
+            , "       uncaught exception: MyException"
+            , "       my exception"
+#if MIN_VERSION_base(4,20,0)
+            , "       HasCallStack backtrace:"
+            , "       "
+#endif
+            , ""
+            , "  To rerun use: --match \"/foo/\" --seed 0"
+            , ""
+            , "Randomized with seed 0"
+            , ""
+            , "Finished in 0.0000 seconds"
+            , "1 example, 1 failure"
+            ]
+
+    context "with --color" $ do
+      it "outputs ColorizedReason" $ do
+        hspecCapture ["--seed=0", "--format=failed-examples", "--color"] $ do
+          H.it "foo" resultWithColorizedReason
+        `shouldReturn` unlines [
+            "\ESC[?25l"
+          , "Failures:"
+          , ""
+          , "  1) foo"
+          , "       some " ++ green "colorized" ++ " error message"
+          , ""
+          , "  To rerun use: --match \"/foo/\" --seed 0"
+          , ""
+          , "Randomized with seed 0"
+          , ""
+          , "Finished in 0.0000 seconds"
+          , red "1 example, 1 failure"
+          , "\ESC[?25h"
+          ]
+
+    context "with --no-color" $ do
+      it "strips ANSI sequences from ColorizedReason" $ do
+        hspecCapture ["--seed=0", "--format=failed-examples", "--no-color"] $ do
+          H.it "foo" resultWithColorizedReason
+        `shouldReturn` unlines [
+            ""
+          , "Failures:"
+          , ""
+          , "  1) foo"
+          , "       some colorized error message"
+          , ""
+          , "  To rerun use: --match \"/foo/\" --seed 0"
+          , ""
+          , "Randomized with seed 0"
+          , ""
+          , "Finished in 0.0000 seconds"
+          , "1 example, 1 failure"
+          ]
+
     context "with --diff" $ do
       it "shows colorized diffs" $ do
         r <- capture_ . ignoreExitCode . withArgs ["--diff", "--color"] . H.hspec $ do
           H.it "foo" $ do
             23 `H.shouldBe` (42 :: Int)
         r `shouldContain` unlines [
-            red ++ "       expected: " ++ reset ++ red ++ "42" ++ reset
-          , red ++ "        but got: " ++ reset ++ green ++ "23" ++ reset
+            red "       expected: " ++ red "42"
+          , red "        but got: " ++ green "23"
           ]
 
     context "with --no-diff" $ do
@@ -532,8 +664,8 @@ spec = do
           H.it "foo" $ do
             23 `H.shouldBe` (42 :: Int)
         r `shouldContain` unlines [
-            red ++ "       expected: " ++ reset ++ "42"
-          , red ++ "        but got: " ++ reset ++ "23"
+            red "       expected: " ++ "42"
+          , red "        but got: " ++ "23"
           ]
 
     context "with --diff-context" $ do
@@ -563,7 +695,7 @@ spec = do
           , "                 @@ 48 lines omitted @@"
           , "                 "
           , ""
-          , "  To rerun use: --match \"/foo/\""
+          , "  To rerun use: --match \"/foo/\" --seed 0"
           , ""
           , "Randomized with seed 0"
           , ""
@@ -597,7 +729,7 @@ spec = do
           , " 52"
           , " 53"
           , ""
-          , "  To rerun use: --match \"/foo/\""
+          , "  To rerun use: --match \"/foo/\" --seed 0"
           , ""
           , "Randomized with seed 0"
           , ""
@@ -607,7 +739,7 @@ spec = do
 
     context "with --print-slow-items" $ do
       it "prints slow items" $ do
-        r <- captureLines . ignoreExitCode . withArgs ["--print-slow-items"] . H.hspec $ do
+        r <- fmap lines . hCapture_ [stdout, stderr] . ignoreExitCode . withArgs ["--print-slow-items"] . H.hspec $ do
           H.it "foo" $ threadDelay 2000
         normalizeTimes (normalizeSummary r) `shouldBe` [
             ""
@@ -617,11 +749,7 @@ spec = do
           , "1 example, 0 failures"
           , ""
           , "Slow spec items:"
-#if MIN_VERSION_base(4,8,1)
-          , "  test" </> "Test" </> "Hspec" </> "Core" </> "RunnerSpec.hs:" <> show (__LINE__ - 10 :: Int) <> ":11: /foo/ (2ms)"
-#else
-          , "  /foo/ (2ms)"
-#endif
+          , "  test" </> "Test" </> "Hspec" </> "Core" </> "RunnerSpec.hs:" <> show (__LINE__ - 9 :: Int) <> ":11: /foo/ (2ms)"
           ]
 
     context "with --format" $ do
@@ -640,25 +768,64 @@ spec = do
           r `shouldContain` "invalid argument `foo' for `--format'"
 
     context "with --qc-max-success" $ do
+      let
+        run :: HasCallStack => String -> IO ()
+        run option = do
+          m <- newMock
+          hspec [option, "23"] $ do
+            H.it "foo" $ property $ \(_ :: Int) -> do
+              mockAction m
+          mockCounter m `shouldReturn` 23
+
       it "tries QuickCheck properties specified number of times" $ do
-        m <- newMock
-        hspec ["--qc-max-success", "23"] $ do
-          H.it "foo" $ property $ \(_ :: Int) -> do
-            mockAction m
-        mockCounter m `shouldReturn` 23
+        run "--qc-max-success"
+
+      it "accepts --maximum-generated-tests as an alias" $ do
+        run "--maximum-generated-tests"
 
       context "when run with --rerun" $ do
         it "takes precedence" $ do
           ["--qc-max-success", "23"] `shouldUseArgs` (QC.maxSuccess, 23)
           ["--rerun", "--qc-max-success", "42"] `shouldUseArgs` (QC.maxSuccess, 42)
 
+    context "with --qc-max-discard" $ do
+      it "passes specified maxDiscardRatio to QuickCheck properties" $ do
+        ["--qc-max-discard", "23"] `shouldUseArgs` (QC.maxDiscardRatio, 23)
+
     context "with --qc-max-size" $ do
-      it "passes specified size to QuickCheck properties" $ do
+      it "passes specified maxSize to QuickCheck properties" $ do
         ["--qc-max-size", "23"] `shouldUseArgs` (QC.maxSize, 23)
 
-    context "with --qc-max-discard" $ do
-      it "uses specified discard ratio to QuickCheck properties" $ do
-        ["--qc-max-discard", "23"] `shouldUseArgs` (QC.maxDiscardRatio, 23)
+    context "with --qc-max-shrinks" $ do
+      it "passes specified maxShrinks to QuickCheck properties" $ do
+        ["--qc-max-shrinks", "23"] `shouldUseArgs` (QC.maxShrinks, 23)
+
+    describe "Test.QuickCheck.Args.chatty" $ do
+      let
+        setChatty value args = args { chatty = value }
+        withChatty value = hspecCapture [] .  H.modifyArgs (setChatty value) $ do
+          H.it "foo" $ once $ property True
+
+      context "when True" $ do
+        it "includes informational output" $ do
+          withChatty True `shouldReturn` unlines [
+              ""
+            , "foo [✔]"
+            , "  +++ OK, passed 1 test."
+            , ""
+            , "Finished in 0.0000 seconds"
+            , "1 example, 0 failures"
+            ]
+
+      context "when False" $ do
+        it "suppresses informational output" $ do
+          withChatty False `shouldReturn` unlines [
+              ""
+            , "foo [✔]"
+            , ""
+            , "Finished in 0.0000 seconds"
+            , "1 example, 0 failures"
+            ]
 
     context "with --seed" $ do
       it "uses specified seed" $ do
@@ -854,7 +1021,3 @@ spec = do
     context "on failure" $ do
       it "returns False" $ do
         H.rerunAll config (Just report) result { specResultSuccess = False } `shouldBe` False
-  where
-    green  = setSGRCode [SetColor Foreground Dull Green]
-    red    = setSGRCode [SetColor Foreground Dull Red]
-    reset  = setSGRCode [Reset]
